@@ -28,6 +28,7 @@ import qualified Data.ByteString.Lazy as L
 import Data.Monoid
 import Control.Monad (void)
 import System.Timeout (timeout)
+import Data.KeyedPool
 
 -- | Perform a @Request@ using a connection acquired from the given @Manager@,
 -- and then provide the @Response@ to the given function. This function is
@@ -90,7 +91,7 @@ httpRaw' req0 m = do
             now <- getCurrentTime
             return $ insertCookiesIntoRequest req' (evictExpiredCookies cj now) now
         Nothing -> return (req', Data.Monoid.mempty)
-    (timeout', (connRelease, ci, isManaged)) <- getConnectionWrapper
+    (timeout', mconn) <- getConnectionWrapper
         (responseTimeout' req)
         (getConn req m)
 
@@ -99,20 +100,20 @@ httpRaw' req0 m = do
     -- connections after accepting the request headers, so we need to check for
     -- exceptions in both.
     ex <- try $ do
-        cont <- requestBuilder (dropProxyAuthSecure req) ci
+        cont <- requestBuilder (dropProxyAuthSecure req) (managedResource mconn)
 
-        getResponse connRelease timeout' req ci cont
+        getResponse timeout' req mconn cont
 
-    case (ex, isManaged) of
+    case ex of
         -- Connection was reused, and might have been closed. Try again
-        (Left e, Reused) | mRetryableException m e -> do
-            connRelease DontReuse
+        Left e | managedReused mconn && mRetryableException m e -> do
+            managedRelease mconn DontReuse
             httpRaw' req m
         -- Not reused, or a non-retry, so this is a real exception
-        (Left e, _) -> throwIO e
+        Left e -> throwIO e
         -- Everything went ok, so the connection is good. If any exceptions get
         -- thrown in the response body, just throw them as normal.
-        (Right res, _) -> case cookieJar req' of
+        Right res -> case cookieJar req' of
             Just _ -> do
                 now' <- getCurrentTime
                 let (cookie_jar, _) = updateCookieJar res req now' cookie_jar'
