@@ -11,6 +11,8 @@ module Network.HTTP.Client.Request
     , parseUrlThrow
     , parseRequest
     , parseRequest_
+    , requestFromURI
+    , requestFromURI_
     , defaultRequest
     , setUriRelative
     , getUri
@@ -25,6 +27,9 @@ module Network.HTTP.Client.Request
     , requestBuilder
     , setRequestIgnoreStatus
     , setQueryString
+#if MIN_VERSION_http_types(0,12,1)
+    , setQueryStringPartialEscape
+#endif
     , streamFile
     , observedStreamFile
     , extractBasicAuthInfo
@@ -53,7 +58,7 @@ import Network.URI (URI (..), URIAuth (..), parseURI, relativeTo, escapeURIStrin
 import Control.Exception (throw, throwIO, IOException)
 import qualified Control.Exception as E
 import qualified Data.CaseInsensitive as CI
-import qualified Data.ByteString.Base64 as B64
+import qualified Data.ByteArray.Encoding as BAE
 
 import Network.HTTP.Client.Body
 import Network.HTTP.Client.Types
@@ -93,8 +98,8 @@ parseUrlThrow =
 
 -- | Convert a URL into a 'Request'.
 --
--- This defaults some of the values in 'Request', such as setting 'method' to
--- GET and 'requestHeaders' to @[]@.
+-- This function defaults some of the values in 'Request', such as setting 'method' to
+-- @"GET"@ and 'requestHeaders' to @[]@.
 --
 -- Since this function uses 'MonadThrow', the return monad can be anything that is
 -- an instance of 'MonadThrow', such as 'IO' or 'Maybe'.
@@ -108,7 +113,7 @@ parseUrlThrow =
 --
 -- Note that the request method must be provided as all capital letters.
 --
--- 'Request' created by this function won't cause exceptions on non-2XX
+-- A 'Request' created by this function won't cause exceptions on non-2XX
 -- response status codes. 
 --
 -- To create a request which throws on non-2XX status codes, see 'parseUrlThrow'
@@ -137,6 +142,29 @@ parseRequest s' =
 parseRequest_ :: String -> Request
 parseRequest_ = either throw id . parseRequest
 
+-- | Convert a 'URI' into a 'Request'.
+--
+-- This can fail if the given 'URI' is not absolute, or if the 
+-- 'URI' scheme is not @"http"@ or @"https"@. In these cases the function
+-- will throw an error via 'MonadThrow'.
+--
+-- This function defaults some of the values in 'Request', such as setting 'method' to
+-- @"GET"@ and 'requestHeaders' to @[]@.
+-- 
+-- A 'Request' created by this function won't cause exceptions on non-2XX
+-- response status codes. 
+--
+-- @since 0.5.12
+requestFromURI :: MonadThrow m => URI -> m Request
+requestFromURI = setUri defaultRequest
+
+-- | Same as 'requestFromURI', but if the conversion would fail,
+-- throws an impure exception.
+--
+-- @since 0.5.12
+requestFromURI_ :: URI -> Request
+requestFromURI_ = either throw id . requestFromURI
+
 -- | Add a 'URI' to the request. If it is absolute (includes a host name), add
 -- it as per 'setUri'; if it is relative, merge it with the existing request.
 setUriRelative :: MonadThrow m => Request -> URI -> m Request
@@ -263,6 +291,14 @@ alwaysDecompress = const True
 browserDecompress :: S.ByteString -> Bool
 browserDecompress = (/= "application/x-tar")
 
+-- | Build a basic-auth header value
+buildBasicAuth ::
+    S8.ByteString -- ^ Username
+    -> S8.ByteString -- ^ Password
+    -> S8.ByteString
+buildBasicAuth user passwd =
+    S8.append "Basic " (BAE.convertToBase BAE.Base64 (S8.concat [ user, ":", passwd ]))
+
 -- | Add a Basic Auth header (with the specified user name and password) to the
 -- given Request. Ignore error handling:
 --
@@ -277,8 +313,7 @@ applyBasicAuth :: S.ByteString -> S.ByteString -> Request -> Request
 applyBasicAuth user passwd req =
     req { requestHeaders = authHeader : requestHeaders req }
   where
-    authHeader = (CI.mk "Authorization", basic)
-    basic = S8.append "Basic " (B64.encode $ S8.concat [ user, ":", passwd ])
+    authHeader = (CI.mk "Authorization", buildBasicAuth user passwd)
 
 -- | Add a proxy to the Request so that the Request when executed will use
 -- the provided proxy.
@@ -299,8 +334,7 @@ applyBasicProxyAuth :: S.ByteString -> S.ByteString -> Request -> Request
 applyBasicProxyAuth user passwd req =
     req { requestHeaders = authHeader : requestHeaders req }
   where
-    authHeader = (CI.mk "Proxy-Authorization", basic)
-    basic = S8.append "Basic " (B64.encode $ S8.concat [ user , ":", passwd ])
+    authHeader = (CI.mk "Proxy-Authorization", buildBasicAuth user passwd)
 
 -- | Add url-encoded parameters to the 'Request'.
 --
@@ -476,6 +510,14 @@ setRequestIgnoreStatus req = req { checkResponse = \_ _ -> return () }
 setQueryString :: [(S.ByteString, Maybe S.ByteString)] -> Request -> Request
 setQueryString qs req = req { queryString = W.renderQuery True qs }
 
+#if MIN_VERSION_http_types(0,12,1)
+-- | Set the query string to the given key/value pairs.
+--
+-- @since 0.5.10
+setQueryStringPartialEscape :: [(S.ByteString, [W.EscapeItem])] -> Request -> Request
+setQueryStringPartialEscape qs req = req { queryString = W.renderQueryPartialEscape True qs }
+#endif
+
 -- | Send a file as the request body.
 --
 -- It is expected that the file size does not change between calling
